==> db-0.0.64.jar: burningswell/db/util.clj <==
(ns burningswell.db.util
  (:refer-clojure :exclude [distinct group-by update])
  (:require [clojure.set :refer [rename-keys]]
            [clojure.string :refer [blank? split]]
            [datumbazo.core :refer :all]
            [no.en.core :refer [parse-double]]
            [geo.core :as geo]
            [geo.postgis :refer [IGeometry geometry point]])
  (:import geo.core.IPoint))

(def embedded-photo
  '(case (is-null :photos.id) nil
         (json_build_object
          "id" :photos.id
          "title" :photos.title
          "flickr"
          (json_build_object
           "id" :photos.flickr-id
           "owner"
           (json_build_object
            "id" :photos.flickr-owner-id
            "name" :photos.flickr-owner-name
            "url" :photos.flickr-owner-url)))))

;; TODO: Whey is geo.core/parse-location not working?
(defn parse-location [s]
  (if (string? s)
    (let [parts (->> (split s #"\s*,\s*")
                     (map parse-double)
                     (remove nil?))]
      (if (= 2 (count parts))
        (apply point 4326 (reverse parts))))
    s))

(defn fulltext
  "Add a where condition to a select query."
  [query & columns]
  (when-not (blank? query)
    (where `(or (~(keyword "@@")
                 (to_tsvector (array_to_string [~@columns], " "))
                 (plainto_tsquery ~query))
                ~@(map #(list 'like %1 (str "%" query "%")) columns))
           :and)))

(defn order-by-distance [geom-1 geom-2]
  (if (and geom-1 geom-2)
    (let [geom-1 (if (string? geom-1) (parse-location geom-1) geom-1)
          geom-2 (if (string? geom-2) (parse-location geom-2) geom-2)]
      (order-by `(st_distance ~(geometry geom-1) ~(geometry geom-2))))))

(defn within-bounding-box
  "Restrict the result set to rows that have `column` within
  `bounding-box`."
  [column bounding-box & [opts]]
  (when bounding-box
    (let [srid (or (:srid opts) 4326)]
      (compose
       (where `(:&&
                ~column
                (cast (st_transform
                       (st_makeenvelope
                        ~(.getX (.getLLB bounding-box))
                        ~(.getY (.getLLB bounding-box))
                        ~(.getX (.getURT bounding-box))
                        ~(.getY (.getURT bounding-box))
                        ~(.getSrid (.getURT bounding-box)))
                       (st_srid ~column))
                      :geography))
              :and)
       (order-by
        `(st_distance
          ~column
          (st_centroid
           (st_transform
            (st_makeenvelope
             ~(.getX (.getLLB bounding-box))
             ~(.getY (.getLLB bounding-box))
             ~(.getX (.getURT bounding-box))
             ~(.getY (.getURT bounding-box))
             ~(.getSrid (.getURT bounding-box)))
            (st_srid ~column)))))))))

(defn within-distance-to
  "Return a WHERE clause where `column` is within `distance` in km to
  `location`."
  [column location distance]
  (when location
    (where `(st_dwithin
             (cast ~(geometry location) :geography)
             (cast ~column :geography)
             ~(* distance 1000)))))

(defn assoc-embedded [m & kv]
  (reduce
   (fn [m [k v]]
     (if-not v
       m (assoc-in m [:_embedded k] v)))
   m (partition 2 kv)))

(defn rename-embedded [m & kvs]
  (reduce
   (fn [m [key [table columns & [url]]]]
     (let [columns (as table columns)
           aliases (map :as columns)
           embedded (->> (reduce #(assoc %1 (:as %2) (:name %2))
                                 {} columns)
                         (rename-keys (select-keys m aliases)))
           vals-nil? (every? nil? (vals embedded))]
       (assoc-embedded
        (apply dissoc m aliases)
        key (if-not vals-nil? embedded))))
   m (partition 2 kvs)))

(extend-type clojure.lang.Keyword
  IGeometry
  (geometry [k]
    k))

(defn zip-by-key
  "Zip the elements of `coll` by their `key`."
  [coll & ks]
  (zipmap (map #(get-in % ks) coll) coll))

(defn zip-by-id
  "Zip the elements of `coll` by their :id key."
  [coll]
  (zip-by-key coll :id))

==> db-0.0.70.jar: burningswell/db/util.clj <==
(ns burningswell.db.util
  (:refer-clojure :exclude [distinct group-by update])
  (:require [clojure.set :refer [rename-keys]]
            [clojure.string :refer [blank? split]]
            [datumbazo.core :refer :all]
            [no.en.core :refer [parse-double]]
            [geo.core :as geo]
            [geo.postgis :refer [IGeometry geometry point]])
  (:import java.awt.Color
           geo.core.IPoint))

(defn color-vector
  "Convert a java.awt.Color into a vector."
  [color]
  [(.getRed color)
   (.getGreen color)
   (.getBlue color)])

(defn geometry-as-png [width height color]
  `(st_aspng
    (st_asraster
     (cast :geom :geometry)
     ~(or width 400)
     ~(or height 400)
     ["8BUI" "8BUI" "8BUI"]
     ;; TODO; Is this really RGB?
     ~(color-vector (or color (Color. 0 0 1)))
     ~(color-vector (Color. 0 0 0)))))

(def embedded-photo
  '(case (is-null :photos.id) nil
         (json_build_object
          "id" :photos.id
          "title" :photos.title
          "flickr"
          (json_build_object
           "id" :photos.flickr-id
           "owner"
           (json_build_object
            "id" :photos.flickr-owner-id
            "name" :photos.flickr-owner-name
            "url" :photos.flickr-owner-url)))))

;; TODO: Whey is geo.core/parse-location not working?
(defn parse-location [s]
  (if (string? s)
    (let [parts (->> (split s #"\s*,\s*")
                     (map parse-double)
                     (remove nil?))]
      (if (= 2 (count parts))
        (apply point 4326 (reverse parts))))
    s))

(defn fulltext
  "Add a where condition to a select query."
  [query & columns]
  (when-not (blank? query)
    (where `(or (~(keyword "@@")
                 (to_tsvector (array_to_string [~@columns], " "))
                 (plainto_tsquery ~query))
                ~@(map #(list 'like %1 (str "%" query "%")) columns))
           :and)))

(defn order-by-distance [geom-1 geom-2]
  (if (and geom-1 geom-2)
    (let [geom-1 (if (string? geom-1) (parse-location geom-1) geom-1)
          geom-2 (if (string? geom-2) (parse-location geom-2) geom-2)]
      (order-by `(st_distance ~(geometry geom-1) ~(geometry geom-2))))))

(defn within-bounding-box
  "Restrict the result set to rows that have `column` within
  `bounding-box`."
  [column bounding-box & [opts]]
  (when bounding-box
    (let [srid (or (:srid opts) 4326)]
      (compose
       (where `(:&&
                ~column
                (cast (st_transform
                       (st_makeenvelope
                        ~(.getX (.getLLB bounding-box))
                        ~(.getY (.getLLB bounding-box))
                        ~(.getX (.getURT bounding-box))
                        ~(.getY (.getURT bounding-box))
                        ~(.getSrid (.getURT bounding-box)))
                       (st_srid ~column))
                      :geography))
              :and)
       (order-by
        `(st_distance
          ~column
          (st_centroid
           (st_transform
            (st_makeenvelope
             ~(.getX (.getLLB bounding-box))
             ~(.getY (.getLLB bounding-box))
             ~(.getX (.getURT bounding-box))
             ~(.getY (.getURT bounding-box))
             ~(.getSrid (.getURT bounding-box)))
            (st_srid ~column)))))))))

(defn within-distance-to
  "Return a WHERE clause where `column` is within `distance` in km to
  `location`."
  [column location distance]
  (when location
    (where `(st_dwithin
             (cast ~(geometry location) :geography)
             (cast ~column :geography)
             ~(* distance 1000)))))

(defn assoc-embedded [m & kv]
  (reduce
   (fn [m [k v]]
     (if-not v
       m (assoc-in m [:_embedded k] v)))
   m (partition 2 kv)))

(defn rename-embedded [m & kvs]
  (reduce
   (fn [m [key [table columns & [url]]]]
     (let [columns (as table columns)
           aliases (map :as columns)
           embedded (->> (reduce #(assoc %1 (:as %2) (:name %2))
                                 {} columns)
                         (rename-keys (select-keys m aliases)))
           vals-nil? (every? nil? (vals embedded))]
       (assoc-embedded
        (apply dissoc m aliases)
        key (if-not vals-nil? embedded))))
   m (partition 2 kvs)))

(extend-type clojure.lang.Keyword
  IGeometry
  (geometry [k]
    k))

(defn zip-by-key
  "Zip the elements of `coll` by their `key`."
  [coll & ks]
  (zipmap (map #(get-in % ks) coll) coll))

(defn zip-by-id
  "Zip the elements of `coll` by their :id key."
  [coll]
  (zip-by-key coll :id))

(defn enabled-gdal-drivers
  "Return the short and long name of the enabled GDAL drivers."
  [db]
  (->> @(select db [(as :short_name :short-name)
                    (as :long_name :long-name)]
          (from (as '(st_gdaldrivers) :drivers))
          (order-by 1))
       (identity)))
